@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2024 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2025 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * https://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -141,20 +141,6 @@ enum class PASS : uint8_t
     obj             // toObjFile() run
 };
 
-enum
-{
-    PASSinit,           // initial state
-    PASSsemantic,       // semantic() started
-    PASSsemanticdone,   // semantic() done
-    PASSsemantic2,      // semantic2() started
-    PASSsemantic2done,  // semantic2() done
-    PASSsemantic3,      // semantic3() started
-    PASSsemantic3done,  // semantic3() done
-    PASSinline,         // inline started
-    PASSinlinedone,     // inline done
-    PASSobj             // toObjFile() run
-};
-
 /* Flags for symbol search
  */
 typedef unsigned SearchOptFlags;
@@ -199,17 +185,22 @@ public:
 #else
     Symbol *csym;               // symbol for code generator
 #endif
-    Loc loc;                    // where defined
     Scope *_scope;               // !=NULL means context to use for semantic()
-    const utf8_t *prettystring;
 private:
     DsymbolAttributes* atts;
 public:
-    d_bool errors;                // this symbol failed to pass semantic()
-    PASS semanticRun;
+    Loc loc;                    // where defined
     unsigned short localNum;        // perturb mangled name to avoid collisions with those in FuncDeclaration.localsymtab
+
+    bool errors() const;
+    PASS semanticRun() const;
+    PASS semanticRun(PASS v);
+private:
+    unsigned char bitfields;
+    unsigned char dsym;
+public:
     static Dsymbol *create(Identifier *);
-    const char *toChars() const override;
+    const char *toChars() const final override;
     DeprecatedDeclaration* depdecl();
     CPPNamespaceDeclaration* cppnamespace();
     UserAttributeDeclaration* userAttribDecl();
@@ -217,8 +208,6 @@ public:
     CPPNamespaceDeclaration* cppnamespace(CPPNamespaceDeclaration* ns);
     UserAttributeDeclaration* userAttribDecl(UserAttributeDeclaration* uad);
     virtual const char *toPrettyCharsHelper(); // helper to print fully qualified (template) arguments
-    Loc getLoc();
-    const char *locToChars();
     bool equals(const RootObject * const o) const override;
     bool isAnonymous() const;
     Module *getModule();
@@ -244,7 +233,7 @@ public:
     virtual Dsymbol *toAlias();                 // resolve real symbol
     virtual Dsymbol *toAlias2();
     virtual bool overloadInsert(Dsymbol *s);
-    virtual uinteger_t size(const Loc &loc);
+    virtual uinteger_t size(Loc loc);
     virtual bool isforwardRef();
     virtual AggregateDeclaration *isThis();     // is a 'this' required to access the member
     virtual bool isExport() const;              // is Dsymbol exported?
@@ -261,11 +250,8 @@ public:
     virtual bool needThis();                    // need a 'this' pointer?
     virtual Visibility visible();
     virtual Dsymbol *syntaxCopy(Dsymbol *s);    // copy only syntax trees
-    virtual bool oneMember(Dsymbol *&ps, Identifier *ident);
     virtual bool hasPointers();
-    virtual bool hasStaticCtorOrDtor();
     virtual void addObjcSymbols(ClassDeclarations *, ClassDeclarations *) { }
-    virtual void checkCtorConstInit() { }
 
     virtual void addComment(const utf8_t *comment);
     const utf8_t *comment();                      // current value of comment
@@ -276,61 +262,61 @@ public:
     bool inNonRoot();
 
     // Eliminate need for dynamic_cast
-    virtual Package *isPackage() { return nullptr; }
-    virtual Module *isModule() { return nullptr; }
-    virtual EnumMember *isEnumMember() { return nullptr; }
-    virtual TemplateDeclaration *isTemplateDeclaration() { return nullptr; }
-    virtual TemplateInstance *isTemplateInstance() { return nullptr; }
-    virtual TemplateMixin *isTemplateMixin() { return nullptr; }
-    virtual ForwardingAttribDeclaration *isForwardingAttribDeclaration() { return nullptr; }
-    virtual Nspace *isNspace() { return nullptr; }
-    virtual Declaration *isDeclaration() { return nullptr; }
-    virtual StorageClassDeclaration *isStorageClassDeclaration(){ return nullptr; }
-    virtual ExpressionDsymbol *isExpressionDsymbol() { return nullptr; }
-    virtual AliasAssign *isAliasAssign() { return nullptr; }
-    virtual ThisDeclaration *isThisDeclaration() { return nullptr; }
-    virtual BitFieldDeclaration *isBitFieldDeclaration() { return nullptr; }
-    virtual TypeInfoDeclaration *isTypeInfoDeclaration() { return nullptr; }
-    virtual TupleDeclaration *isTupleDeclaration() { return nullptr; }
-    virtual AliasDeclaration *isAliasDeclaration() { return nullptr; }
-    virtual AggregateDeclaration *isAggregateDeclaration() { return nullptr; }
-    virtual FuncDeclaration *isFuncDeclaration() { return nullptr; }
-    virtual FuncAliasDeclaration *isFuncAliasDeclaration() { return nullptr; }
-    virtual OverDeclaration *isOverDeclaration() { return nullptr; }
-    virtual FuncLiteralDeclaration *isFuncLiteralDeclaration() { return nullptr; }
-    virtual CtorDeclaration *isCtorDeclaration() { return nullptr; }
-    virtual PostBlitDeclaration *isPostBlitDeclaration() { return nullptr; }
-    virtual DtorDeclaration *isDtorDeclaration() { return nullptr; }
-    virtual StaticCtorDeclaration *isStaticCtorDeclaration() { return nullptr; }
-    virtual StaticDtorDeclaration *isStaticDtorDeclaration() { return nullptr; }
-    virtual SharedStaticCtorDeclaration *isSharedStaticCtorDeclaration() { return nullptr; }
-    virtual SharedStaticDtorDeclaration *isSharedStaticDtorDeclaration() { return nullptr; }
-    virtual InvariantDeclaration *isInvariantDeclaration() { return nullptr; }
-    virtual UnitTestDeclaration *isUnitTestDeclaration() { return nullptr; }
-    virtual NewDeclaration *isNewDeclaration() { return nullptr; }
-    virtual VarDeclaration *isVarDeclaration() { return nullptr; }
-    virtual VersionSymbol *isVersionSymbol() { return nullptr; }
-    virtual DebugSymbol *isDebugSymbol() { return nullptr; }
-    virtual ClassDeclaration *isClassDeclaration() { return nullptr; }
-    virtual StructDeclaration *isStructDeclaration() { return nullptr; }
-    virtual UnionDeclaration *isUnionDeclaration() { return nullptr; }
-    virtual InterfaceDeclaration *isInterfaceDeclaration() { return nullptr; }
-    virtual ScopeDsymbol *isScopeDsymbol() { return nullptr; }
-    virtual ForwardingScopeDsymbol *isForwardingScopeDsymbol() { return nullptr; }
-    virtual WithScopeSymbol *isWithScopeSymbol() { return nullptr; }
-    virtual ArrayScopeSymbol *isArrayScopeSymbol() { return nullptr; }
-    virtual Import *isImport() { return nullptr; }
-    virtual EnumDeclaration *isEnumDeclaration() { return nullptr; }
-    virtual SymbolDeclaration *isSymbolDeclaration() { return nullptr; }
-    virtual AttribDeclaration *isAttribDeclaration() { return nullptr; }
-    virtual AnonDeclaration *isAnonDeclaration() { return nullptr; }
-    virtual CPPNamespaceDeclaration *isCPPNamespaceDeclaration() { return nullptr; }
-    virtual VisibilityDeclaration *isVisibilityDeclaration() { return nullptr; }
-    virtual OverloadSet *isOverloadSet() { return nullptr; }
-    virtual MixinDeclaration *isMixinDeclaration() { return nullptr; }
-    virtual StaticAssert *isStaticAssert() { return nullptr; }
-    virtual StaticIfDeclaration *isStaticIfDeclaration() { return nullptr; }
-    virtual CAsmDeclaration *isCAsmDeclaration() { return nullptr; }
+    Package *isPackage();
+    Module *isModule();
+    EnumMember *isEnumMember();
+    TemplateDeclaration *isTemplateDeclaration();
+    TemplateInstance *isTemplateInstance();
+    TemplateMixin *isTemplateMixin();
+    ForwardingAttribDeclaration *isForwardingAttribDeclaration();
+    Nspace *isNspace();
+    Declaration *isDeclaration();
+    StorageClassDeclaration *isStorageClassDeclaration();
+    ExpressionDsymbol *isExpressionDsymbol();
+    AliasAssign *isAliasAssign();
+    ThisDeclaration *isThisDeclaration();
+    BitFieldDeclaration *isBitFieldDeclaration();
+    TypeInfoDeclaration *isTypeInfoDeclaration();
+    TupleDeclaration *isTupleDeclaration();
+    AliasDeclaration *isAliasDeclaration();
+    AggregateDeclaration *isAggregateDeclaration();
+    FuncDeclaration *isFuncDeclaration();
+    FuncAliasDeclaration *isFuncAliasDeclaration();
+    OverDeclaration *isOverDeclaration();
+    FuncLiteralDeclaration *isFuncLiteralDeclaration();
+    CtorDeclaration *isCtorDeclaration();
+    PostBlitDeclaration *isPostBlitDeclaration();
+    DtorDeclaration *isDtorDeclaration();
+    StaticCtorDeclaration *isStaticCtorDeclaration();
+    StaticDtorDeclaration *isStaticDtorDeclaration();
+    SharedStaticCtorDeclaration *isSharedStaticCtorDeclaration();
+    SharedStaticDtorDeclaration *isSharedStaticDtorDeclaration();
+    InvariantDeclaration *isInvariantDeclaration();
+    UnitTestDeclaration *isUnitTestDeclaration();
+    NewDeclaration *isNewDeclaration();
+    VarDeclaration *isVarDeclaration();
+    VersionSymbol *isVersionSymbol();
+    DebugSymbol *isDebugSymbol();
+    ClassDeclaration *isClassDeclaration();
+    StructDeclaration *isStructDeclaration();
+    UnionDeclaration *isUnionDeclaration();
+    InterfaceDeclaration *isInterfaceDeclaration();
+    ScopeDsymbol *isScopeDsymbol();
+    ForwardingScopeDsymbol *isForwardingScopeDsymbol();
+    WithScopeSymbol *isWithScopeSymbol();
+    ArrayScopeSymbol *isArrayScopeSymbol();
+    Import *isImport();
+    EnumDeclaration *isEnumDeclaration();
+    SymbolDeclaration *isSymbolDeclaration();
+    AttribDeclaration *isAttribDeclaration();
+    AnonDeclaration *isAnonDeclaration();
+    CPPNamespaceDeclaration *isCPPNamespaceDeclaration();
+    VisibilityDeclaration *isVisibilityDeclaration();
+    OverloadSet *isOverloadSet();
+    MixinDeclaration *isMixinDeclaration();
+    StaticAssert *isStaticAssert();
+    StaticIfDeclaration *isStaticIfDeclaration();
+    CAsmDeclaration *isCAsmDeclaration();
     void accept(Visitor *v) override { v->visit(this); }
 };
 
@@ -353,13 +339,11 @@ public:
     virtual void importScope(Dsymbol *s, Visibility visibility);
     virtual bool isPackageAccessible(Package *p, Visibility visibility, SearchOptFlags flags = (SearchOptFlags)SearchOpt::all);
     bool isforwardRef() override final;
-    static void multiplyDefined(const Loc &loc, Dsymbol *s1, Dsymbol *s2);
+    static void multiplyDefined(Loc loc, Dsymbol *s1, Dsymbol *s2);
     const char *kind() const override;
     virtual Dsymbol *symtabInsert(Dsymbol *s);
     virtual Dsymbol *symtabLookup(Dsymbol *s, Identifier *id);
-    bool hasStaticCtorOrDtor() override;
 
-    ScopeDsymbol *isScopeDsymbol() override final { return this; }
     void accept(Visitor *v) override { v->visit(this); }
 };
 
@@ -371,7 +355,6 @@ public:
     WithStatement *withstate;
 
 
-    WithScopeSymbol *isWithScopeSymbol() override { return this; }
     void accept(Visitor *v) override { v->visit(this); }
 };
 
@@ -382,7 +365,6 @@ class ArrayScopeSymbol final : public ScopeDsymbol
 public:
     RootObject *arrayContent;
 
-    ArrayScopeSymbol *isArrayScopeSymbol() override { return this; }
     void accept(Visitor *v) override { v->visit(this); }
 };
 
@@ -394,7 +376,6 @@ public:
     Dsymbols a;         // array of Dsymbols
 
     void push(Dsymbol *s);
-    OverloadSet *isOverloadSet() override { return this; }
     const char *kind() const override;
     void accept(Visitor *v) override { v->visit(this); }
 };
@@ -409,7 +390,6 @@ public:
     void importScope(Dsymbol *s, Visibility visibility) override;
     const char *kind() const override;
 
-    ForwardingScopeDsymbol *isForwardingScopeDsymbol() override { return this; }
 };
 
 class ExpressionDsymbol final : public Dsymbol
@@ -417,7 +397,6 @@ class ExpressionDsymbol final : public Dsymbol
 public:
     Expression *exp;
 
-    ExpressionDsymbol *isExpressionDsymbol() override { return this; }
 };
 
 class CAsmDeclaration final : public Dsymbol
@@ -425,7 +404,6 @@ class CAsmDeclaration final : public Dsymbol
 public:
     Expression *code;   // string expression
 
-    CAsmDeclaration *isCAsmDeclaration() override { return this; }
     void accept(Visitor *v) override { v->visit(this); }
 };
 
@@ -453,7 +431,11 @@ public:
 namespace dmd
 {
     void addMember(Dsymbol *dsym, Scope *sc, ScopeDsymbol *sds);
-    Dsymbol *search(Dsymbol *d, const Loc &loc, Identifier *ident, SearchOptFlags flags = (SearchOptFlags)SearchOpt::localsOnly);
+    Dsymbol *search(Dsymbol *d, Loc loc, Identifier *ident, SearchOptFlags flags = (SearchOptFlags)SearchOpt::localsOnly);
+    Dsymbols *include(Dsymbol *d, Scope *sc);
     void setScope(Dsymbol *d, Scope *sc);
     void importAll(Dsymbol *d, Scope *sc);
+    void addComment(Dsymbol *d, const char *comment);
+    bool oneMember(Dsymbol *d, Dsymbol *&ps, Identifier *ident);
+    bool hasStaticCtorOrDtor(Dsymbol *d);
 }
