@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2024 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2025 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * https://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -20,7 +20,7 @@ class Expression;
 namespace dmd
 {
     // in enumsem.d
-    Expression *getDefaultValue(EnumDeclaration *ed, const Loc &loc);
+    Expression *getDefaultValue(EnumDeclaration *ed, Loc loc);
 }
 
 class EnumDeclaration final : public ScopeDsymbol
@@ -52,14 +52,12 @@ public:
     bool inuse(bool v);
 
     EnumDeclaration *syntaxCopy(Dsymbol *s) override;
-    bool oneMember(Dsymbol *&ps, Identifier *ident) override;
     Type *getType() override;
     const char *kind() const override;
     bool isDeprecated() const override;       // is Dsymbol deprecated?
     Visibility visible() override;
     bool isSpecial() const;
 
-    EnumDeclaration *isEnumDeclaration() override { return this; }
 
 #if !IN_LLVM
     Symbol *sinit;
@@ -89,6 +87,5 @@ public:
     EnumMember *syntaxCopy(Dsymbol *s) override;
     const char *kind() const override;
 
-    EnumMember *isEnumMember() override { return this; }
     void accept(Visitor *v) override { v->visit(this); }
 };
