@@ -73,20 +73,12 @@ static std::error_code createSymLink(const char *to, const char *from) {
 #include <windows.h>
 namespace llvm {
 namespace sys {
-#if LDC_LLVM_VER >= 1100
 namespace windows {
 // Fwd declaration to an internal LLVM function.
 std::error_code widenPath(const llvm::Twine &Path8,
                           llvm::SmallVectorImpl<wchar_t> &Path16,
                           size_t MaxPathLen = MAX_PATH);
 }
-#else
-namespace path {
-// Fwd declaration to an internal LLVM function.
-std::error_code widenPath(const llvm::Twine &Path8,
-                          llvm::SmallVectorImpl<wchar_t> &Path16);
-}
-#endif // LDC_LLVM_VER < 1100
 } // namespace sys
 } // namespace llvm
 
@@ -100,11 +92,7 @@ std::error_code createLink(FType f, const char *to, const char *from) {
   //
   //===----------------------------------------------------------------------===//
 
-#if LDC_LLVM_VER >= 1100
   using llvm::sys::windows::widenPath;
-#else
-  using llvm::sys::path::widenPath;
-#endif
 
   llvm::SmallVector<wchar_t, 128> wide_from;
   llvm::SmallVector<wchar_t, 128> wide_to;
@@ -321,14 +309,30 @@ void outputIR2ObjRelevantCmdlineArgs(llvm::raw_ostream &hash_os) {
   hash_os << opts::getFeaturesStr();
   hash_os << opts::floatABI;
   const auto relocModel = opts::getRelocModel();
+#if LDC_LLVM_VER >= 1600
+  if (relocModel.has_value())
+    hash_os << relocModel.value();
+#else
   if (relocModel.hasValue())
     hash_os << relocModel.getValue();
+#endif
   const auto codeModel = opts::getCodeModel();
+#if LDC_LLVM_VER >= 1600
+  if (codeModel.has_value())
+    hash_os << codeModel.value();
+#else
   if (codeModel.hasValue())
     hash_os << codeModel.getValue();
+#endif
+
   const auto framePointerUsage = opts::framePointerUsage();
+#if LDC_LLVM_VER >= 1600
+  if (framePointerUsage.has_value())
+    hash_os << static_cast<int>(framePointerUsage.value());
+#else
   if (framePointerUsage.hasValue())
     hash_os << static_cast<int>(framePointerUsage.getValue());
+#endif
 }
 
 // Output to `hash_os` all environment flags that influence object code output
