@@ -21,19 +21,10 @@
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/Support/SpecialCaseList.h"
-#if LDC_LLVM_VER >= 1300
 #include "llvm/Support/VirtualFileSystem.h"
-#endif
-
-#if LDC_LLVM_VER >= 1400
 #include "llvm/Transforms/Instrumentation/AddressSanitizerOptions.h"
-#else
-namespace llvm {
-// Declaring this simplifies code later, but the option is never used with LLVM
-// <= 13.
-enum class AsanDetectStackUseAfterReturnMode { Never, Runtime, Always };
-}
-#endif
+
+using namespace dmd;
 
 namespace {
 
@@ -59,6 +50,12 @@ cl::list<std::string> fSanitizeCoverage(
 
 llvm::SanitizerCoverageOptions sanitizerCoverageOptions;
 
+cl::list<std::string>
+    fSanitizeRecover("fsanitize-recover", cl::CommaSeparated,
+                     cl::desc("Enable recovery for specified sanitizers ('all' "
+                              "= enable for all supported sanitizers)."),
+                     cl::value_desc("name"));
+
 SanitizerBits parseFSanitizeCmdlineParameter() {
   SanitizerBits retval = 0;
   for (const auto &name : fSanitize) {
@@ -70,6 +67,24 @@ SanitizerBits parseFSanitizeCmdlineParameter() {
   return retval;
 }
 
+SanitizerBits parseFSanitizeRecoverCmdlineParameter() {
+  SanitizerBits retval = 0;
+  for (const auto &name : fSanitizeRecover) {
+    if (name == "all") {
+      retval |= ~SanitizerBits(0);
+      continue;
+    }
+    SanitizerCheck check = parseSanitizerName(name, [&name] {
+      error(Loc(), "fsanitize-recover: Unrecognized sanitizer name '%s'.", name.c_str());
+    });
+    if (check & supportedSanitizerRecoveries)
+      retval |= SanitizerBits(check);
+    else
+      error(Loc(), "fsanitize-recover: Recovery not supported for '%s'.", name.c_str());
+  }
+  return retval;
+}
+
 void parseFSanitizeCoverageParameter(llvm::StringRef name,
                                      llvm::SanitizerCoverageOptions &opts) {
   if (name == "func") {
@@ -95,14 +110,12 @@ void parseFSanitizeCoverageParameter(llvm::StringRef name,
   else if (name == "trace-gep") {
     opts.TraceGep = true;
   }
-#if LDC_LLVM_VER >= 1400
   else if (name == "trace-loads") {
     opts.TraceLoads = true;
   }
   else if (name == "trace-stores") {
     opts.TraceStores = true;
   }
-#endif
   else if (name == "8bit-counters") {
     opts.Use8bitCounters = true;
   }
@@ -159,6 +172,8 @@ cl::opt<llvm::AsanDetectStackUseAfterReturnMode> fSanitizeAddressUseAfterReturn(
             "but not as much as never). Requires druntime support.")));
 
 SanitizerBits enabledSanitizers = 0;
+SanitizerBits enabledSanitizerRecoveries = 0;
+const SanitizerBits supportedSanitizerRecoveries = AddressSanitizer | MemorySanitizer;
 
 // Parse sanitizer name passed on commandline and return the corresponding
 // sanitizer bits.
@@ -182,6 +197,7 @@ SanitizerCheck parseSanitizerName(llvm::StringRef name,
 void initializeSanitizerOptionsFromCmdline()
 {
   enabledSanitizers |= parseFSanitizeCmdlineParameter();
+  enabledSanitizerRecoveries |= parseFSanitizeRecoverCmdlineParameter();
 
   auto &sancovOpts = sanitizerCoverageOptions;
 
