@@ -113,7 +113,7 @@ bool TargetABI::isAggregate(Type *t) {
   //        by runtime functions, for which we don't apply the rewrites yet
   //        when calling them
   return ty == TY::Tstruct || ty == TY::Tsarray ||
-         /*ty == TY::Tarray ||*/ ty == TY::Tdelegate || t->iscomplex();
+         /*ty == TY::Tarray ||*/ ty == TY::Tdelegate || t->isComplex();
 }
 
 bool TargetABI::isPOD(Type *t, bool excludeStructsWithCtor) {
@@ -155,15 +155,48 @@ llvm::CallingConv::ID TargetABI::callingConv(FuncDeclaration *fdecl) {
   return callingConv(tf, fdecl->needThis() || fdecl->isNested());
 }
 
+void TargetABI::setUnwindTableKind(llvm::Function *fn) {
+  llvm::UWTableKind kind = defaultUnwindTableKind();
+  if (kind != llvm::UWTableKind::None) {
+#if LDC_LLVM_VER >= 1600
+    fn->setUWTableKind(kind);
+#else
+    fn->setUWTableKind(kind);
+#endif
+  }
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
+bool TargetABI::returnInArg(TypeFunction *tf, bool needsThis) {
+  // default: use sret for non-PODs or if a same-typed argument would be passed
+  // byval
+  Type *rt = tf->next->toBasetype();
+  return !isPOD(rt) || passByVal(tf, rt);
+}
+
 bool TargetABI::preferPassByRef(Type *t) {
   // simple base heuristic: use a ref for all types > 2 machine words
   return size(t) > 2 * target.ptrsize;
 }
 
+bool TargetABI::passByVal(TypeFunction *tf, Type *t) {
+  // default: all POD structs and static arrays
+  return DtoIsInMemoryOnly(t) && isPOD(t);
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
+void TargetABI::rewriteFunctionType(IrFuncTy &fty) {
+  if (!skipReturnValueRewrite(fty))
+    rewriteArgument(fty, *fty.ret);
+
+  for (auto arg : fty.args) {
+    if (!arg->byref)
+      rewriteArgument(fty, *arg);
+  }
+}
+
 void TargetABI::rewriteVarargs(IrFuncTy &fty,
                                std::vector<IrFuncTyArg *> &args) {
   for (auto arg : args) {
@@ -173,6 +206,15 @@ void TargetABI::rewriteVarargs(IrFuncTy &fty,
   }
 }
 
+void TargetABI::rewriteArgument(IrFuncTy &fty,
+                               IrFuncTyArg &arg) {
+  // default: pass non-PODs indirectly by-value
+  if (!isPOD(arg.type)) {
+    static IndirectByvalRewrite indirectByvalRewrite;
+    indirectByvalRewrite.applyTo(arg);
+  }
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 LLValue *TargetABI::prepareVaStart(DLValue *ap) {
@@ -213,32 +255,6 @@ const char *TargetABI::objcMsgSendFunc(Type *ret, IrFuncTy &fty, bool directcall
 
 //////////////////////////////////////////////////////////////////////////////
 
-// Some reasonable defaults for when we don't know what ABI to use.
-struct UnknownTargetABI : TargetABI {
-  bool returnInArg(TypeFunction *tf, bool) override {
-    if (tf->isref()) {
-      return false;
-    }
-
-    // Return structs and static arrays on the stack. The latter is needed
-    // because otherwise LLVM tries to actually return the array in a number
-    // of physical registers, which leads, depending on the target, to
-    // either horrendous codegen or backend crashes.
-    Type *rt = tf->next->toBasetype();
-    return passByVal(tf, rt);
-  }
-
-  bool passByVal(TypeFunction *, Type *t) override {
-    return DtoIsInMemoryOnly(t);
-  }
-
-  void rewriteFunctionType(IrFuncTy &) override {
-    // why?
-  }
-};
-
-//////////////////////////////////////////////////////////////////////////////
-
 TargetABI *TargetABI::getTarget() {
   switch (global.params.targetTriple->getArch()) {
   case llvm::Triple::x86:
@@ -277,8 +293,10 @@ TargetABI *TargetABI::getTarget() {
   case llvm::Triple::wasm64:
     return getWasmTargetABI();
   default:
-    Logger::cout() << "WARNING: Unknown ABI, guessing...\n";
-    return new UnknownTargetABI;
+    warning(Loc(),
+            "unknown target ABI, falling back to generic implementation. C/C++ "
+            "interop will almost certainly NOT work.");
+    return new TargetABI;
   }
 }
 
@@ -297,6 +315,7 @@ struct IntrinsicABI : TargetABI {
     if (ty->ty != TY::Tstruct) {
       return;
     }
+    assert(isPOD(arg.type));
     // TODO: Check that no unions are passed in or returned.
 
     LLType *abiTy = DtoUnpaddedStructType(arg.type);
@@ -305,32 +324,6 @@ struct IntrinsicABI : TargetABI {
       remove_padding.applyTo(arg, abiTy);
     }
   }
-
-  void rewriteFunctionType(IrFuncTy &fty) override {
-    if (!fty.arg_sret) {
-      Type *rt = fty.ret->type->toBasetype();
-      if (rt->ty == TY::Tstruct) {
-        Logger::println("Intrinsic ABI: Transforming return type");
-        rewriteArgument(fty, *fty.ret);
-      }
-    }
-
-    Logger::println("Intrinsic ABI: Transforming arguments");
-    LOG_SCOPE;
-
-    for (auto arg : fty.args) {
-      IF_LOG Logger::cout() << "Arg: " << arg->type->toChars() << '\n';
-
-      // Arguments that are in memory are of no interest to us.
-      if (arg->byref) {
-        continue;
-      }
-
-      rewriteArgument(fty, *arg);
-
-      IF_LOG Logger::cout() << "New arg type: " << *arg->ltype << '\n';
-    }
-  }
 };
 
 TargetABI *TargetABI::getIntrinsic() {
