@@ -86,7 +86,7 @@ struct HardfloatRewrite : BaseBitcastABIRewrite {
     LLType *t[2];
     for (unsigned i = 0; i < 2; ++i) {
       t[i] =
-          flat.fields[i]->isfloating()
+          flat.fields[i]->isFloating()
               ? DtoType(flat.fields[i])
               : LLIntegerType::get(gIR->context(), size(flat.fields[i]) * 8);
     }
@@ -101,7 +101,6 @@ struct HardfloatRewrite : BaseBitcastABIRewrite {
 struct LoongArch64TargetABI : TargetABI {
 private:
   HardfloatRewrite hardfloatRewrite;
-  IndirectByvalRewrite indirectByvalRewrite{};
   Integer2Rewrite integer2Rewrite;
   IntegerRewrite integerRewrite;
 
@@ -112,61 +111,39 @@ private:
     if (result.length <= 0)
       return false;
     if (result.length == 1)
-      return result.fields[0]->isfloating();
-    return result.fields[0]->isfloating() || result.fields[1]->isfloating();
+      return result.fields[0]->isFloating();
+    return result.fields[0]->isFloating() || result.fields[1]->isFloating();
   }
 
 public:
-  auto returnInArg(TypeFunction *tf, bool) -> bool override {
-    if (tf->isref()) {
-      return false;
-    }
-    Type *rt = tf->next->toBasetype();
-    if (!isPOD(rt)) {
-      return true;
-    }
-    // pass by reference when > 2*GRLEN
-    return size(rt) > 16;
+  bool passByVal(TypeFunction *, Type *t) override {
+    return isPOD(t) && size(t) > 16;
   }
 
-  auto passByVal(TypeFunction *, Type *t) -> bool override {
-    if (!isPOD(t)) {
-      return false;
-    }
-    return size(t) > 16;
-  }
-
-  void rewriteFunctionType(IrFuncTy &fty) override {
-    if (!skipReturnValueRewrite(fty)) {
-      if (requireHardfloatRewrite(fty.ret->type)) {
-        // rewrite here because we should not apply this to variadic arguments
-        hardfloatRewrite.applyTo(*fty.ret);
-      } else {
-        rewriteArgument(fty, *fty.ret);
-      }
-    }
-
-    for (auto arg : fty.args) {
-      if (!arg->byref) {
-        if (requireHardfloatRewrite(arg->type)) {
-          // rewrite here because we should not apply this to variadic arguments
-          hardfloatRewrite.applyTo(*arg);
-        } else {
-          rewriteArgument(fty, *arg);
-        }
-      }
+  void rewriteVarargs(IrFuncTy &fty,
+                      std::vector<IrFuncTyArg *> &args) override {
+    for (auto arg : args) {
+      if (!arg->byref)
+        rewriteArgument(fty, *arg, /*isVararg=*/true);
     }
   }
 
   void rewriteArgument(IrFuncTy &fty, IrFuncTyArg &arg) override {
-    if (!isPOD(arg.type)) {
-      // non-PODs should be passed in memory
-      indirectByvalRewrite.applyTo(arg);
+    rewriteArgument(fty, arg, /*isVararg=*/false);
+  }
+
+  void rewriteArgument(IrFuncTy &fty, IrFuncTyArg &arg, bool isVararg) {
+    TargetABI::rewriteArgument(fty, arg);
+    if (arg.rewrite)
+      return;
+
+    if (!isVararg && requireHardfloatRewrite(arg.type)) {
+      hardfloatRewrite.applyTo(arg);
       return;
     }
 
     Type *ty = arg.type->toBasetype();
-    if (ty->isintegral() && (ty->ty == TY::Tint32 || ty->ty == TY::Tuns32 ||
+    if (ty->isIntegral() && (ty->ty == TY::Tint32 || ty->ty == TY::Tuns32 ||
                              ty->ty == TY::Tdchar)) {
       // In the LP64D ABI, both int32 and unsigned int32 are stored in
       // general-purpose registers as proper sign extensions of their
