@@ -93,8 +93,8 @@ bool requireHardfloatRewrite(Type *ty) {
   if (result.length <= 0)
     return false;
   if (result.length == 1)
-    return result.fields[0].ty->isfloating();
-  return result.fields[0].ty->isfloating() || result.fields[1].ty->isfloating();
+    return result.fields[0].ty->isFloating();
+  return result.fields[0].ty->isFloating() || result.fields[1].ty->isFloating();
 }
 
 struct HardfloatRewrite : ABIRewrite {
@@ -139,7 +139,7 @@ struct HardfloatRewrite : ABIRewrite {
     assert(flat.length == 2);
     LLType *t[2];
     for (unsigned i = 0; i < 2; ++i) {
-      t[i] = flat.fields[i].ty->isfloating()
+      t[i] = flat.fields[i].ty->isFloating()
                  ? DtoType(flat.fields[i].ty)
                  : LLIntegerType::get(gIR->context(),
                                       size(flat.fields[i].ty) * 8);
@@ -158,56 +158,47 @@ private:
   IntegerRewrite integerRewrite;
 
 public:
+  llvm::UWTableKind defaultUnwindTableKind() override {
+    return global.params.targetTriple->isOSLinux() ? llvm::UWTableKind::Async
+                                                   : llvm::UWTableKind::None;
+  }
+
   Type *vaListType() override {
     // va_list is void*
     return pointerTo(Type::tvoid);
   }
   bool returnInArg(TypeFunction *tf, bool) override {
-    if (tf->isref()) {
-      return false;
-    }
     Type *rt = tf->next->toBasetype();
-    if (!size(rt))
-      return false;
-    if (!isPOD(rt))
-      return true;
-    return size(rt) > 16;
+    return !isPOD(rt) || size(rt) > 16;
   }
   bool passByVal(TypeFunction *, Type *t) override {
-    if (!size(t))
-      return false;
-    if (t->toBasetype()->ty == TY::Tcomplex80) {
+    t = t->toBasetype();
+    if (t->ty == TY::Tcomplex80) {
       // rewrite it later to bypass the RVal problem
       return false;
     }
-    return size(t) > 16;
+    return isPOD(t) && size(t) > 16;
   }
-  void rewriteFunctionType(IrFuncTy &fty) override {
-    if (!fty.ret->byref) {
-      if (!skipReturnValueRewrite(fty)) {
-        if (!fty.ret->byref && isPOD(fty.ret->type) &&
-            requireHardfloatRewrite(fty.ret->type)) {
-          // rewrite here because we should not apply this to variadic arguments
-          hardfloatRewrite.applyTo(*fty.ret);
-        } else {
-          rewriteArgument(fty, *fty.ret);
-        }
-      }
-    }
 
-    for (auto arg : fty.args) {
-      if (!arg->byref && isPOD(arg->type) &&
-          requireHardfloatRewrite(arg->type)) {
-        // rewrite here because we should not apply this to variadic arguments
-        hardfloatRewrite.applyTo(*arg);
-      } else {
-        rewriteArgument(fty, *arg);
-      }
+  void rewriteVarargs(IrFuncTy &fty,
+                      std::vector<IrFuncTyArg *> &args) override {
+    for (auto arg : args) {
+      if (!arg->byref)
+        rewriteArgument(fty, *arg, /*isVararg=*/true);
     }
   }
 
   void rewriteArgument(IrFuncTy &fty, IrFuncTyArg &arg) override {
-    if (arg.byref) {
+    rewriteArgument(fty, arg, /*isVararg=*/false);
+  }
+
+  void rewriteArgument(IrFuncTy &fty, IrFuncTyArg &arg, bool isVararg) {
+    TargetABI::rewriteArgument(fty, arg);
+    if (arg.rewrite)
+      return;
+
+    if (!isVararg && requireHardfloatRewrite(arg.type)) {
+      hardfloatRewrite.applyTo(arg);
       return;
     }
 
@@ -218,12 +209,6 @@ public:
       return;
     }
 
-    if (!isPOD(arg.type)) {
-      // non-PODs should be passed in memory
-      indirectByvalRewrite.applyTo(arg);
-      return;
-    }
-
     if (isAggregate(ty) && size(ty) && size(ty) <= 16) {
       if (size(ty) > 8 && DtoAlignment(ty) < 16) {
         // pass the aggregate as {int64, int64} to avoid wrong alignment
