@@ -146,6 +146,10 @@ struct X86_64TargetABI : TargetABI {
   ImplicitByvalRewrite byvalRewrite;
   IndirectByvalRewrite indirectByvalRewrite;
 
+  llvm::UWTableKind defaultUnwindTableKind() override {
+    return llvm::UWTableKind::Async;
+  }
+
   bool returnInArg(TypeFunction *tf, bool needsThis) override;
 
   bool preferPassByRef(Type *t) override;
@@ -197,10 +201,6 @@ private:
 };
 
 bool X86_64TargetABI::returnInArg(TypeFunction *tf, bool) {
-  if (tf->isref()) {
-    return false;
-  }
-
   Type *rt = tf->next->toBasetype();
 
   // x87 creal is returned on the x87 stack
@@ -387,7 +387,7 @@ const char *X86_64TargetABI::objcMsgSendFunc(Type *ret, IrFuncTy &fty, bool dire
     return directcall ? "objc_msgSendSuper_stret" : "objc_msgSend_stret";
   }
   // float, double, long double return
-  if (ret && ret->isfloating()) {
+  if (ret && ret->isFloating()) {
     return ret->ty == TY::Tcomplex80 ? "objc_msgSend_fp2ret" : "objc_msgSend_fpret";
   }
   return directcall ? "objc_msgSendSuper" : "objc_msgSend";
