@@ -147,13 +147,13 @@ DValue *isAssociativeArrayAndNull(Type *type, LLValue *lhs, LLValue *rhs) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-DValue *binAdd(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binAdd(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   Type *lhsType = lhs->type->toBasetype();
   Type *rhsType = rhs->type->toBasetype();
 
   if (lhsType != rhsType && lhsType->ty == TY::Tpointer &&
-      rhsType->isintegral()) {
+      rhsType->isIntegral()) {
     Logger::println("Adding integer to pointer");
     return emitPointerOffset(loc, lhs, rhs, false, type, loadLhsAfterRhs);
   }
@@ -162,7 +162,7 @@ DValue *binAdd(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
   if (type->ty == TY::Tnull)
     return DtoNullValue(type, loc);
-  if (type->iscomplex())
+  if (type->isComplex())
     return DtoComplexAdd(loc, type, rvals.lhs, rvals.rhs);
 
   LLValue *l = DtoRVal(DtoCast(loc, rvals.lhs, type));
@@ -171,7 +171,7 @@ DValue *binAdd(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
   if (auto aa = isAssociativeArrayAndNull(type, l, r))
     return aa;
 
-  LLValue *res = (type->isfloating() ? gIR->ir->CreateFAdd(l, r)
+  LLValue *res = (type->isFloating() ? gIR->ir->CreateFAdd(l, r)
                                      : gIR->ir->CreateAdd(l, r));
 
   return new DImValue(type, res);
@@ -179,13 +179,13 @@ DValue *binAdd(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
 //////////////////////////////////////////////////////////////////////////////
 
-DValue *binMin(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binMin(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   Type *lhsType = lhs->type->toBasetype();
   Type *rhsType = rhs->type->toBasetype();
 
   if (lhsType != rhsType && lhsType->ty == TY::Tpointer &&
-      rhsType->isintegral()) {
+      rhsType->isIntegral()) {
     Logger::println("Subtracting integer from pointer");
     return emitPointerOffset(loc, lhs, rhs, true, type, loadLhsAfterRhs);
   }
@@ -207,7 +207,7 @@ DValue *binMin(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
   if (type->ty == TY::Tnull)
     return DtoNullValue(type, loc);
-  if (type->iscomplex())
+  if (type->isComplex())
     return DtoComplexMin(loc, type, rvals.lhs, rvals.rhs);
 
   LLValue *l = DtoRVal(DtoCast(loc, rvals.lhs, type));
@@ -216,7 +216,7 @@ DValue *binMin(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
   if (auto aa = isAssociativeArrayAndNull(type, l, r))
     return aa;
 
-  LLValue *res = (type->isfloating() ? gIR->ir->CreateFSub(l, r)
+  LLValue *res = (type->isFloating() ? gIR->ir->CreateFSub(l, r)
                                      : gIR->ir->CreateSub(l, r));
 
   return new DImValue(type, res);
@@ -224,16 +224,16 @@ DValue *binMin(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
 //////////////////////////////////////////////////////////////////////////////
 
-DValue *binMul(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binMul(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   auto rvals = evalSides(lhs, rhs, loadLhsAfterRhs);
 
-  if (type->iscomplex())
+  if (type->isComplex())
     return DtoComplexMul(loc, type, rvals.lhs, rvals.rhs);
 
   LLValue *l = DtoRVal(DtoCast(loc, rvals.lhs, type));
   LLValue *r = DtoRVal(DtoCast(loc, rvals.rhs, type));
-  LLValue *res = (type->isfloating() ? gIR->ir->CreateFMul(l, r)
+  LLValue *res = (type->isFloating() ? gIR->ir->CreateFMul(l, r)
                                      : gIR->ir->CreateMul(l, r));
 
   return new DImValue(type, res);
@@ -241,17 +241,17 @@ DValue *binMul(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
 //////////////////////////////////////////////////////////////////////////////
 
-DValue *binDiv(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binDiv(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   auto rvals = evalSides(lhs, rhs, loadLhsAfterRhs);
 
-  if (type->iscomplex())
+  if (type->isComplex())
     return DtoComplexDiv(loc, type, rvals.lhs, rvals.rhs);
 
   LLValue *l = DtoRVal(DtoCast(loc, rvals.lhs, type));
   LLValue *r = DtoRVal(DtoCast(loc, rvals.rhs, type));
   LLValue *res;
-  if (type->isfloating()) {
+  if (type->isFloating()) {
     res = gIR->ir->CreateFDiv(l, r);
   } else if (!isLLVMUnsigned(type)) {
     res = gIR->ir->CreateSDiv(l, r);
@@ -264,17 +264,17 @@ DValue *binDiv(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
 //////////////////////////////////////////////////////////////////////////////
 
-DValue *binMod(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binMod(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   auto rvals = evalSides(lhs, rhs, loadLhsAfterRhs);
 
-  if (type->iscomplex())
+  if (type->isComplex())
     return DtoComplexMod(loc, type, rvals.lhs, rvals.rhs);
 
   LLValue *l = DtoRVal(DtoCast(loc, rvals.lhs, type));
   LLValue *r = DtoRVal(DtoCast(loc, rvals.rhs, type));
   LLValue *res;
-  if (type->isfloating()) {
+  if (type->isFloating()) {
     res = gIR->ir->CreateFRem(l, r);
   } else if (!isLLVMUnsigned(type)) {
     res = gIR->ir->CreateSRem(l, r);
@@ -288,7 +288,7 @@ DValue *binMod(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 //////////////////////////////////////////////////////////////////////////////
 
 namespace {
-DValue *binBitwise(llvm::Instruction::BinaryOps binOp, const Loc &loc,
+DValue *binBitwise(llvm::Instruction::BinaryOps binOp, Loc loc,
                    Type *type, DValue *lhs, Expression *rhs,
                    bool loadLhsAfterRhs) {
   auto rvals = evalSides(lhs, rhs, loadLhsAfterRhs);
@@ -301,38 +301,38 @@ DValue *binBitwise(llvm::Instruction::BinaryOps binOp, const Loc &loc,
 }
 }
 
-DValue *binAnd(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binAnd(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   return binBitwise(llvm::Instruction::And, loc, type, lhs, rhs,
                     loadLhsAfterRhs);
 }
 
-DValue *binOr(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binOr(Loc loc, Type *type, DValue *lhs, Expression *rhs,
               bool loadLhsAfterRhs) {
   return binBitwise(llvm::Instruction::Or, loc, type, lhs, rhs,
                     loadLhsAfterRhs);
 }
 
-DValue *binXor(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binXor(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   return binBitwise(llvm::Instruction::Xor, loc, type, lhs, rhs,
                     loadLhsAfterRhs);
 }
 
-DValue *binShl(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binShl(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   return binBitwise(llvm::Instruction::Shl, loc, type, lhs, rhs,
                     loadLhsAfterRhs);
 }
 
-DValue *binShr(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binShr(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                bool loadLhsAfterRhs) {
   auto op = (isLLVMUnsigned(type) ? llvm::Instruction::LShr
                                   : llvm::Instruction::AShr);
   return binBitwise(op, loc, type, lhs, rhs, loadLhsAfterRhs);
 }
 
-DValue *binUshr(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
+DValue *binUshr(Loc loc, Type *type, DValue *lhs, Expression *rhs,
                 bool loadLhsAfterRhs) {
   return binBitwise(llvm::Instruction::LShr, loc, type, lhs, rhs,
                     loadLhsAfterRhs);
@@ -340,18 +340,18 @@ DValue *binUshr(const Loc &loc, Type *type, DValue *lhs, Expression *rhs,
 
 //////////////////////////////////////////////////////////////////////////////
 
-LLValue *DtoBinNumericEquals(const Loc &loc, DValue *lhs, DValue *rhs, EXP op) {
+LLValue *DtoBinNumericEquals(Loc loc, DValue *lhs, DValue *rhs, EXP op) {
   assert(op == EXP::equal || op == EXP::notEqual || op == EXP::identity ||
          op == EXP::notIdentity);
   Type *t = lhs->type->toBasetype();
-  assert(t->isfloating());
+  assert(t->isFloating());
   Logger::println("numeric equality");
 
   LLValue *res = nullptr;
-  if (t->iscomplex()) {
+  if (t->isComplex()) {
     Logger::println("complex");
     res = DtoComplexEquals(loc, op, lhs, rhs);
-  } else if (t->isfloating()) {
+  } else if (t->isFloating()) {
     Logger::println("floating");
     res = DtoBinFloatsEquals(loc, lhs, rhs, op);
   }
@@ -362,7 +362,7 @@ LLValue *DtoBinNumericEquals(const Loc &loc, DValue *lhs, DValue *rhs, EXP op) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-LLValue *DtoBinFloatsEquals(const Loc &loc, DValue *lhs, DValue *rhs, EXP op) {
+LLValue *DtoBinFloatsEquals(Loc loc, DValue *lhs, DValue *rhs, EXP op) {
   LLValue *res = nullptr;
   if (op == EXP::equal || op == EXP::notEqual) {
     LLValue *l = DtoRVal(lhs);
