@@ -28,16 +28,16 @@ class TypeClass;
 /// Resolves the llvm type for a class declaration
 void DtoResolveClass(ClassDeclaration *cd);
 
-DValue *DtoNewClass(const Loc &loc, TypeClass *type, NewExp *newexp);
+DValue *DtoNewClass(Loc loc, TypeClass *type, NewExp *newexp);
 void DtoInitClass(TypeClass *tc, llvm::Value *dst);
-void DtoFinalizeClass(const Loc &loc, llvm::Value *inst);
-void DtoFinalizeScopeClass(const Loc &loc, DValue *dval,
+void DtoFinalizeClass(Loc loc, llvm::Value *inst);
+void DtoFinalizeScopeClass(Loc loc, DValue *dval,
                            bool dynTypeMatchesStaticType);
 
-DValue *DtoCastClass(const Loc &loc, DValue *val, Type *to);
-DValue *DtoDynamicCastObject(const Loc &loc, DValue *val, Type *to);
+DValue *DtoCastClass(Loc loc, DValue *val, Type *to);
+DValue *DtoDynamicCastObject(Loc loc, DValue *val, Type *to);
 
-DValue *DtoDynamicCastInterface(const Loc &loc, DValue *val, Type *to);
+DValue *DtoDynamicCastInterface(Loc loc, DValue *val, Type *to);
 
 bool DtoIsObjcLinkage(Type *to);
 
