@@ -14,7 +14,7 @@
 #include "gen/irstate.h"
 #include "gen/logger.h"
 
-void emitCoverageLinecountInc(const Loc &loc) {
+void emitCoverageLinecountInc(Loc loc) {
   Module *m = gIR->dmodule;
 
   // Only emit coverage increment for locations in the source of the current
