@@ -18,13 +18,12 @@
 struct DComputePointerRewrite : ABIRewrite {
   LLValue *put(DValue *v, bool isLValueExp, bool) override {
     LLValue *address = DtoLVal(v);
-    address = DtoGEP(DtoType(v->type), address, 0u, 0u);
     return DtoLoad(type(v->type), address, ".DComputePointerRewrite_arg");
   }
 
   LLValue *getLVal(Type *dty, LLValue *v) override {
     LLValue *mem = DtoAlloca(dty, ".DComputePointerRewrite_param_storage");
-    DtoStore(v, DtoGEP(DtoType(dty), mem, 0u, 0u));
+    DtoStore(v, mem);
     return mem;
   }
 
