@@ -117,7 +117,7 @@ DIBuilder::DIBuilder(IRState *const IR)
       // (https://reviews.llvm.org/D23720)
       emitColumnInfo(opts::getFlagOrDefault(::emitColumnInfo, !emitCodeView)) {}
 
-unsigned DIBuilder::getColumn(const Loc &loc) const {
+unsigned DIBuilder::getColumn(Loc loc) const {
   return (loc.linnum() && emitColumnInfo) ? loc.charnum() : 0;
 }
 
@@ -202,7 +202,7 @@ llvm::StringRef DIBuilder::GetNameAndScope(Dsymbol *sym, DIScope &scope) {
 }
 
 // Sets the memory address for a debuginfo variable.
-void DIBuilder::Declare(const Loc &loc, llvm::Value *storage,
+void DIBuilder::Declare(Loc loc, llvm::Value *storage,
                         DILocalVariable divar, DIExpression diexpr) {
   auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum(),
                                         getColumn(loc), GetCurrentScope());
@@ -210,7 +210,7 @@ void DIBuilder::Declare(const Loc &loc, llvm::Value *storage,
 }
 
 // Sets the (current) value for a debuginfo variable.
-void DIBuilder::SetValue(const Loc &loc, llvm::Value *value,
+void DIBuilder::SetValue(Loc loc, llvm::Value *value,
                          DILocalVariable divar, DIExpression diexpr) {
   auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum(),
                                         getColumn(loc), GetCurrentScope());
@@ -241,7 +241,7 @@ DIFile DIBuilder::CreateFile(const char *filename) {
   return DBuilder.createFile(filename, cwd);
 }
 
-DIFile DIBuilder::CreateFile(const Loc &loc) {
+DIFile DIBuilder::CreateFile(Loc loc) {
   return CreateFile(loc.filename());
 }
 
@@ -353,7 +353,7 @@ DIType DIBuilder::CreateEnumType(TypeEnum *type) {
 
   // just emit a typedef for non-integral base types
   auto tb = type->toBasetype();
-  if (!tb->isintegral()) {
+  if (!tb->isIntegral()) {
     auto tbase = CreateTypeDescription(tb);
     return DBuilder.createTypedef(tbase, name, file, lineNumber, scope);
   }
@@ -509,7 +509,7 @@ void DIBuilder::AddStaticMembers(AggregateDeclaration *ad, DIFile file,
   std::function<void(Dsymbols *)> visitMembers = [&](Dsymbols *members) {
     for (auto s : *members) {
       if (auto attrib = s->isAttribDeclaration()) {
-        if (Dsymbols *d = attrib->include(nullptr))
+        if (Dsymbols *d = include(attrib, nullptr))
           visitMembers(d);
       } else if (auto tmixin = s->isTemplateMixin()) {
         // FIXME: static variables inside a template mixin need to be put inside
@@ -720,16 +720,56 @@ DIType DIBuilder::CreateAArrayType(TypeAArray *type) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DISubroutineType DIBuilder::CreateFunctionType(Type *type) {
+DISubroutineType DIBuilder::CreateFunctionType(Type *type,
+                                               FuncDeclaration *fd) {
   TypeFunction *t = type->isTypeFunction();
   assert(t);
 
-  Type *retType = t->next;
+  llvm::SmallVector<LLMetadata *, 8> params;
+  auto pushParam = [&](Type *type, bool isRef) {
+    auto ditype = CreateTypeDescription(type);
+    if (isRef) {
+      if (!ditype) { // void or noreturn
+        ditype = CreateTypeDescription(Type::tuns8);
+      }
+      ditype = DBuilder.createReferenceType(llvm::dwarf::DW_TAG_reference_type,
+                                            ditype, target.ptrsize * 8);
+    }
+    params.emplace_back(ditype);
+  };
+
+  // the first 'param' is the return value
+  pushParam(t->next, t->isRef());
+
+  // then the implicit 'this'/context pointer
+  if (fd) {
+    DIType pointeeType = nullptr;
+    if (auto parentAggregate = fd->isThis()) {
+      pointeeType = CreateCompositeType(parentAggregate->type);
+    } else if (fd->isNested()) {
+      pointeeType = CreateTypeDescription(Type::tuns8); // cannot use void
+    }
+
+    if (pointeeType) {
+      DIType ditype = DBuilder.createReferenceType(
+          llvm::dwarf::DW_TAG_pointer_type, pointeeType, target.ptrsize * 8);
+      ditype = DBuilder.createObjectPointerType(ditype
+#if LDC_LLVM_VER >= 2000
+      , /* Implicit */ true
+#endif
+      );
+      params.emplace_back(ditype);
+    }
+  }
+
+  // and finally the formal parameters
+  const auto len = t->parameterList.length();
+  for (size_t i = 0; i < len; i++) {
+    const auto param = t->parameterList[i];
+    pushParam(param->type, param->isReference());
+  }
 
-  // Create "dummy" subroutine type for the return type
-  LLMetadata *params = {CreateTypeDescription(retType)};
   auto paramsArray = DBuilder.getOrCreateTypeArray(params);
-
   return DBuilder.createSubroutineType(paramsArray, DIFlags::FlagZero, 0);
 }
 
@@ -786,7 +826,7 @@ DIType DIBuilder::CreateTypeDescription(Type *t, bool voidToUbyte) {
     return CreateEnumType(te);
   if (auto tv = t->isTypeVector())
     return CreateVectorType(tv);
-  if (t->isintegral() || t->isfloating())
+  if (t->isIntegral() || t->isFloating())
     return CreateBasicType(t);
   if (auto tp = t->isTypePointer())
     return CreatePointerType(tp);
@@ -971,7 +1011,7 @@ DISubprogram DIBuilder::EmitSubProgram(FuncDeclaration *fd) {
         flags, dispFlags);
 
     // Now create subroutine type.
-    diFnType = CreateFunctionType(fd->type);
+    diFnType = CreateFunctionType(fd->type, fd);
   }
 
   // FIXME: duplicates?
@@ -1010,7 +1050,7 @@ DISubprogram DIBuilder::EmitThunk(llvm::Function *Thunk, FuncDeclaration *fd) {
          "Compilation unit missing or corrupted in DIBuilder::EmitThunk");
 
   // Create subroutine type (thunk has same type as wrapped function)
-  DISubroutineType DIFnType = CreateFunctionType(fd->type);
+  DISubroutineType DIFnType = CreateFunctionType(fd->type, fd);
 
   const auto scope = GetSymbolScope(fd);
   const auto name = (llvm::Twine(fd->toChars()) + ".__thunk").str();
@@ -1077,7 +1117,7 @@ void DIBuilder::EmitFuncStart(FuncDeclaration *fd) {
   EmitStopPoint(fd->loc);
 }
 
-void DIBuilder::EmitBlockStart(const Loc &loc) {
+void DIBuilder::EmitBlockStart(Loc loc) {
   if (!mustEmitLocationsDebugInfo())
     return;
 
@@ -1102,7 +1142,7 @@ void DIBuilder::EmitBlockEnd() {
   fn->diLexicalBlocks.pop();
 }
 
-void DIBuilder::EmitStopPoint(const Loc &loc) {
+void DIBuilder::EmitStopPoint(Loc loc) {
   if (!mustEmitLocationsDebugInfo())
     return;
 
