@@ -158,7 +158,7 @@ bool defineAsExternallyAvailable(FuncDeclaration &fdecl) {
 
   IF_LOG Logger::println("Potential inlining candidate");
 
-  if (fdecl.semanticRun < PASS::semantic3) {
+  if (fdecl.semanticRun() < PASS::semantic3) {
     IF_LOG Logger::println("Do semantic analysis");
     LOG_SCOPE
 
@@ -180,7 +180,7 @@ bool defineAsExternallyAvailable(FuncDeclaration &fdecl) {
       IF_LOG Logger::println("Errors occured during semantic analysis.");
       return false;
     }
-    assert(fdecl.semanticRun >= PASS::semantic3done);
+    assert(fdecl.semanticRun() >= PASS::semantic3done);
   }
 
   // FuncDeclaration::naked is set by the AsmParser during semantic3 analysis,
