@@ -41,7 +41,7 @@ void DtoDeclareFunction(FuncDeclaration *fdecl);
 void DtoDefineFunction(FuncDeclaration *fd, bool linkageAvailableExternally = false);
 
 void DtoDefineNakedFunction(FuncDeclaration *fd);
-void emitABIReturnAsmStmt(IRAsmBlock *asmblock, const Loc &loc,
+void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc loc,
                           FuncDeclaration *fdecl);
 
 DValue *DtoArgument(Parameter *fnarg, Expression *argexp);
