@@ -114,10 +114,4 @@ void print(const char *fmt, ...) {
     va_end(va);
   }
 }
-void attention(const Loc &loc, const char *fmt, ...) {
-  va_list va;
-  va_start(va, fmt);
-  vwarning(loc, fmt, va);
-  va_end(va);
-}
 }
