@@ -100,7 +100,7 @@ std::string hashSymbolName(llvm::StringRef name, Dsymbol *symb) {
 
 std::string getIRMangledName(FuncDeclaration *fdecl, LINK link) {
   std::string mangledName = mangleExact(fdecl);
-  if (fdecl->adFlags & 4) { // nounderscore
+  if (fdecl->noUnderscore()) {
     mangledName.insert(0, "\1");
   }
 
