@@ -27,12 +27,12 @@ class Type;
 bool initRuntime();
 void freeRuntime();
 
-llvm::Function *getRuntimeFunction(const Loc &loc, llvm::Module &target,
+llvm::Function *getRuntimeFunction(Loc loc, llvm::Module &target,
                                    const char *name);
 
-llvm::Function *getCAssertFunction(const Loc &loc, llvm::Module &target);
+llvm::Function *getCAssertFunction(Loc loc, llvm::Module &target);
 
-llvm::Function *getUnwindResumeFunction(const Loc &loc, llvm::Module &target);
+llvm::Function *getUnwindResumeFunction(Loc loc, llvm::Module &target);
 
 void emitInstrumentationFnEnter(FuncDeclaration *decl);
 void emitInstrumentationFnLeave(FuncDeclaration *decl);
