@@ -26,6 +26,20 @@
 using namespace dmd;
 using llvm::APFloat;
 
+enum class RealPrecision : uint8_t { Default, Double, Quad, DoubleDouble };
+static llvm::cl::opt<RealPrecision, false> realPrecision{
+    "real-precision",
+    llvm::cl::ZeroOrMore,
+    llvm::cl::Hidden,
+    llvm::cl::init(RealPrecision::Default),
+    llvm::cl::desc("Override the precision of the `real` type"),
+    llvm::cl::values(clEnumValN(RealPrecision::Double, "double",
+                                "Use double precision (64-bit)"),
+                     clEnumValN(RealPrecision::Quad, "quad",
+                                "Use IEEE quad precision (128-bit)"),
+                     clEnumValN(RealPrecision::DoubleDouble, "doubledouble",
+                                "Use IBM double double precision (128-bit)"))};
+
 namespace {
 // Returns the LL type to be used for D `real` (C `long double`).
 llvm::Type *getRealType(const llvm::Triple &triple) {
@@ -33,6 +47,21 @@ llvm::Type *getRealType(const llvm::Triple &triple) {
 
   auto &ctx = getGlobalContext();
 
+  // if overridden with -real-precision:
+  if (realPrecision == RealPrecision::Double) {
+    return LLType::getDoubleTy(ctx);
+  } else if (realPrecision == RealPrecision::Quad) {
+    return LLType::getFP128Ty(ctx);
+  } else if (realPrecision == RealPrecision::DoubleDouble) {
+    if (triple.getArch() != Triple::ppc64 &&
+        triple.getArch() != Triple::ppc64le) {
+      error(Loc(), "'-real-precision=doubledouble' is only supported for "
+                   "PowerPC64 targets");
+      fatal();
+    }
+    return LLType::getPPC_FP128Ty(ctx);
+  }
+
   // Android: x86 targets follow ARM, with emulated quad precision for x64
   if (triple.getEnvironment() == llvm::Triple::Android) {
     return triple.isArch64Bit() ? LLType::getFP128Ty(ctx)
@@ -48,9 +77,9 @@ llvm::Type *getRealType(const llvm::Triple &triple) {
 
   case Triple::aarch64:
   case Triple::aarch64_be:
-    // AArch64 has 128-bit quad precision; Apple uses double
-    return triple.isOSDarwin() ? LLType::getDoubleTy(ctx)
-                               : LLType::getFP128Ty(ctx);
+    // AArch64 has 128-bit quad precision; Apple and MSVC use double
+    return triple.isOSDarwin() || triple.isWindowsMSVCEnvironment()
+               ? LLType::getDoubleTy(ctx) : LLType::getFP128Ty(ctx);
 
   case Triple::riscv32:
   case Triple::riscv64:
@@ -64,9 +93,34 @@ llvm::Type *getRealType(const llvm::Triple &triple) {
   case Triple::wasm64:
     return LLType::getFP128Ty(ctx);
 
+  case Triple::ppc64:
+  case Triple::ppc64le:
+    if (triple.isMusl()) { // Musl uses double
+      return LLType::getDoubleTy(ctx);
+    }
+#if defined(__linux__) && defined(__powerpc64__)
+    // for a PowerPC64 Linux build:
+    // default to the C++ host compiler's `long double` ABI when targeting
+    // PowerPC64 (non-musl) Linux
+    if (triple.isOSLinux()) {
+#if __LDBL_MANT_DIG__ == 113
+      return LLType::getFP128Ty(ctx);
+#elif __LDBL_MANT_DIG__ == 106
+      return LLType::getPPC_FP128Ty(ctx);
+#elif __LDBL_MANT_DIG__ == 53
+      return LLType::getDoubleTy(ctx);
+#else
+      static_assert(
+          __LDBL_MANT_DIG__ == 0,
+          "Unexpected C++ 'long double' precision for a PowerPC64 host!");
+#endif
+    }
+#endif
+    return LLType::getPPC_FP128Ty(ctx);
+
   default:
     // 64-bit double precision for all other targets
-    // FIXME: PowerPC, SystemZ, ...
+    // FIXME: SystemZ, ...
     return LLType::getDoubleTy(ctx);
   }
 }
@@ -156,6 +210,7 @@ void Target::_init(const Param &params) {
   const auto IEEEdouble = &APFloat::IEEEdouble();
   const auto x87DoubleExtended = &APFloat::x87DoubleExtended();
   const auto IEEEquad = &APFloat::IEEEquad();
+  const auto PPCDoubleDouble = &APFloat::PPCDoubleDouble();
   bool isOutOfRange = false;
 
   RealProperties.nan = CTFloat::nan;
@@ -197,6 +252,18 @@ void Target::_init(const Param &params) {
     RealProperties.min_exp = -16381;
     RealProperties.max_10_exp = 4932;
     RealProperties.min_10_exp = -4931;
+  } else if (targetRealSemantics == PPCDoubleDouble) {
+    RealProperties.max =
+        CTFloat::parse("0x1.fffffffffffff7ffffffffffff8p1023", isOutOfRange);
+    RealProperties.min_normal = CTFloat::parse("0x1p-969", isOutOfRange);
+    RealProperties.epsilon =
+        CTFloat::parse("0x0.000000000000000000000000008p-969", isOutOfRange);
+    RealProperties.dig = 31;
+    RealProperties.mant_dig = 106;
+    RealProperties.max_exp = 1024;
+    RealProperties.min_exp = -968;
+    RealProperties.max_10_exp = 308;
+    RealProperties.min_10_exp = -291;
   } else {
     // leave initialized with host real_t values
     warning(Loc(), "unknown properties for target `real` type, relying on D "
@@ -220,7 +287,7 @@ unsigned Target::alignsize(Type *type) {
  */
 unsigned Target::fieldalign(Type *type) { return DtoAlignment(type); }
 
-Type *Target::va_listType(const Loc &loc, Scope *sc) {
+Type *Target::va_listType(Loc loc, Scope *sc) {
   if (!tvalist)
     tvalist = typeSemantic(gABI->vaListType(), loc, sc);
   return tvalist;
@@ -237,11 +304,29 @@ Type *Target::va_listType(const Loc &loc, Scope *sc) {
 const char *TargetCPP::typeMangle(Type *t) {
   if (t->ty == TY::Tfloat80) {
     const auto &triple = *global.params.targetTriple;
+
     // `long double` on Android/x64 is __float128 and mangled as `g`
-    bool isAndroidX64 = triple.getEnvironment() == llvm::Triple::Android &&
-                        triple.getArch() == llvm::Triple::x86_64;
-    return isAndroidX64 ? "g" : "e";
+    if (triple.getEnvironment() == llvm::Triple::Android &&
+        triple.getArch() == llvm::Triple::x86_64 &&
+        target.RealProperties.mant_dig == 113) {
+      return "g";
+    };
+
+    if (triple.getArch() == llvm::Triple::ppc64 ||
+        triple.getArch() == llvm::Triple::ppc64le) {
+      if (target.RealProperties.mant_dig == 113 &&
+          triple.getEnvironment() == llvm::Triple::GNU) {
+        return "u9__ieee128";
+      }
+      if (target.RealProperties.mant_dig == 106) {
+        // IBM long double
+        return "g";
+      }
+    }
+
+    return "e";
   }
+
   return nullptr;
 }
 
@@ -258,12 +343,12 @@ TypeTuple *Target::toArgTypes(Type *t) {
 }
 
 bool Target::isReturnOnStack(TypeFunction *tf, bool needsThis) {
-  return gABI->returnInArg(tf, needsThis);
+  return !tf->isRef() && gABI->returnInArg(tf, needsThis);
 }
 
 bool Target::preferPassByRef(Type *t) { return gABI->preferPassByRef(t); }
 
-Expression *Target::getTargetInfo(const char *name_, const Loc &loc) {
+Expression *Target::getTargetInfo(const char *name_, Loc loc) {
   const llvm::StringRef name(name_);
   const auto &triple = *global.params.targetTriple;
 
