@@ -118,7 +118,7 @@ public:
     } else {
       assert(llvm::isa<LLIntegerType>(t));
       result = LLConstantInt::get(t, static_cast<uint64_t>(e->getInteger()),
-                                  !e->type->isunsigned());
+                                  !e->type->isUnsigned());
       assert(result);
       IF_LOG Logger::cout() << "value = " << *result << '\n';
     }
@@ -190,7 +190,7 @@ public:
   // very similar to emitPointerOffset() in binops.cpp
   LLConstant *tryEmitPointerOffset(BinExp *e, bool negateOffset) {
     Type *t1b = e->e1->type->toBasetype();
-    if (t1b->ty != TY::Tpointer || !e->e2->type->isintegral())
+    if (t1b->ty != TY::Tpointer || !e->e2->type->isIntegral())
       return nullptr;
 
     Type *const pointeeType = t1b->nextOf();
@@ -392,40 +392,6 @@ public:
     IF_LOG Logger::println("AddrExp::toConstElem: %s @ %s", e->toChars(),
                            e->type->toChars());
     LOG_SCOPE;
-    // FIXME: this should probably be generalized more so we don't
-    // need to have a case for each thing we can take the address of
-
-    // address of global variable
-    if (auto vexp = e->e1->isVarExp()) {
-      result = DtoConstSymbolAddress(e->loc, vexp->var);
-      return;
-    }
-
-    // address of indexExp
-    if (auto iexp = e->e1->isIndexExp()) {
-      // indexee must be global static array var
-      VarExp *vexp = iexp->e1->isVarExp();
-      assert(vexp);
-      VarDeclaration *vd = vexp->var->isVarDeclaration();
-      assert(vd);
-      assert(vd->type->toBasetype()->ty == TY::Tsarray);
-      DtoResolveVariable(vd);
-      assert(isIrGlobalCreated(vd));
-
-      // get index
-      LLConstant *index = toConstElem(iexp->e2, p);
-      assert(index->getType() == DtoSize_t());
-
-      // gep
-      LLConstant *idxs[2] = {DtoConstSize_t(0), index};
-      LLConstant *val = isaConstant(getIrGlobal(vd)->value);
-      LLConstant *gep = llvm::ConstantExpr::getGetElementPtr(
-          DtoType(vd->type), val, idxs, true);
-
-      assert(e->type->toBasetype()->ty == TY::Tpointer);
-      result = gep;
-      return;
-    }
 
     if (auto se = e->e1->isStructLiteralExp()) {
       result = p->getStructLiteralGlobal(se);
@@ -449,12 +415,7 @@ public:
       return;
     }
 
-    if (e->e1->op == EXP::slice || e->e1->op == EXP::dotVariable) {
-      visit(static_cast<Expression *>(e));
-      return;
-    }
-
-    llvm_unreachable("unsupported AddrExp in ToConstElemVisitor");
+    visit(static_cast<Expression *>(e));
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -545,7 +506,7 @@ public:
                          e->toChars(), e->type ? e->type->toChars() : "(null)");
     LOG_SCOPE;
 
-    if (e->useStaticInit) {
+    if (e->useStaticInit()) {
       DtoResolveStruct(e->sd);
       result = getIrAggr(e->sd)->getDefaultInit();
     } else {
