@@ -60,7 +60,7 @@
 
 using namespace dmd;
 
-TypeInfoDeclaration *getOrCreateTypeInfoDeclaration(const Loc &loc, Type *forType) {
+TypeInfoDeclaration *getOrCreateTypeInfoDeclaration(Loc loc, Type *forType) {
   IF_LOG Logger::println("getOrCreateTypeInfoDeclaration(): %s",
                          forType->toChars());
   LOG_SCOPE
@@ -148,7 +148,7 @@ public:
 
     // void[] init
     // the array is null if the default initializer is zero
-    if (!sd->members || decl->tinfo->isZeroInit(decl->loc)) {
+    if (!sd->members || isZeroInit(decl->tinfo, decl->loc)) {
       b.push_null_void_array();
     }
     // otherwise emit a void[] with the default initializer
@@ -231,6 +231,9 @@ public:
     // key typeinfo
     b.push_typeinfo(tc->index);
 
+    // entry typeinfo (key-value pair)
+    b.push_typeinfo(decl->entry);
+
     // finish
     b.finalize(gvar);
   }
