@@ -16,6 +16,7 @@
 
 #include "driver/cl_options.h"
 #include "gen/irstate.h"
+#include "gen/llvm.h"
 
 #include <vector>
 #include <utility>
@@ -30,8 +31,8 @@ static llvm::cl::opt<bool> fEmitLocalVarLifetime(
 LocalVariableLifetimeAnnotator::LocalVariableLifetimeAnnotator(IRState &irs)
     : irs(irs) {
   allocaType =
-      llvm::Type::getInt8Ty(irs.context())
-          ->getPointerTo(irs.module.getDataLayout().getAllocaAddrSpace());
+      LLPointerType::get(LLType::getInt8Ty(irs.context()),
+                         irs.module.getDataLayout().getAllocaAddrSpace());
 }
 
 void LocalVariableLifetimeAnnotator::pushScope() { scopes.emplace_back(); }
