@@ -12,7 +12,11 @@ void *loadSym(void *handle, const char *name) { return GetProcAddress(handle, na
 
 #include <dlfcn.h>
 
+#if __APPLE__
+#define LIB_SO "lib.dylib"
+#else
 #define LIB_SO "lib.so"
+#endif
 
 void *loadLib(const char *name) { return dlopen(name, RTLD_LAZY); }
 int closeLib(void *handle) { return dlclose(handle) == 0 ? 1 : 0; }
