@@ -11,8 +11,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef UTILS_HPP
-#define UTILS_HPP
+#pragma once
 
 #include <string>
 
@@ -25,5 +24,3 @@ void fatal(const Context &context, const std::string &reason);
 void interruptPoint(const Context &context, const char *desc,
                     const char *object = "");
 void verifyModule(const Context &context, llvm::Module &module);
-
-#endif // UTILS_HPP
