@@ -1,7 +1,10 @@
 // check dsymbolSemantic analysis of C files
 /* TEST_OUTPUT:
+REQUIRED_ARGS: -verrors=context
 ---
 fail_compilation/failcstuff6.c(56): Error: enum member `failcstuff6.test_overflow.boom` initialization with `2147483647+1` causes overflow for type `int`
+    boom,
+        ^
 ---
 */
 
