@@ -1,7 +1,7 @@
 /*
 TEST_OUTPUT:
 ---
-fail_compilation/test22070.c(10): Error: `&""` is not an lvalue and cannot be modified
+fail_compilation/test22070.c(10): Error: cannot take address of expression `&""` because it is not an lvalue
 ---
 */
 
